import os

import cv2
import numpy as np

from skcvideo.reader import WHITE, put_text


class Button(object):
    def __init__(self, hitbox, callback, data=None):
        self.hitbox = hitbox
        self.data = data
        self.given_callback = callback

    def callback(self, *kwargs):
        if self.data is None:
            return self.given_callback(*kwargs)
        else:
            return self.given_callback(self.data, *kwargs)


class ImageGenerator(object):
    def __init__(self, images_dir):
        self.to_exit = False

        self.keydict = {'k': self.next, 'j': self.previous, 'q': self.exit}

        self.buttons = []

        self.images_list = []
        for filename in os.listdir(images_dir):
            ext = os.path.splitext(filename)[1]
            if ext in ['.png', '.jpg']:
                image_path = os.path.join(images_dir, filename)
                image = cv2.imread(image_path)
                self.images_list.append(image)

        self.i = 0
        self.refresh()

        cv2.namedWindow('image')
        cv2.setMouseCallback('image', self.click_event)

    @property
    def image_to_disp(self):
        return self.big_image

    def next(self):
        if self.i < len(self.images_list) - 1:
            self.i += 1
            self.refresh()

    def previous(self):
        if self.i > 0:
            self.i -= 1
            self.refresh()

    def exit(self):
        self.to_exit = True

    def click_event(self, event, x, y, flags, param):
        if event == cv2.EVENT_LBUTTONUP:
            if hasattr(self, 'buttons'):
                for button in self.buttons:
                    x1, y1, x2, y2 = button.hitbox
                    if x1 < x < x2 and y1 < y < y2:
                        button.callback(x, y)
            self.refresh()

    def refresh(self):
        self.big_image = np.zeros((150, 150, 3), dtype=np.uint8)

        self.image = self.images_list[self.i]

        put_text(self.big_image, 'i: {}'.format(self.i), (20, 30), WHITE)

        self.big_image[:, :75, :] = self.image

    def start(self):
        while not self.to_exit:
            cv2.imshow('image', self.image_to_disp)
            key = cv2.waitKey(1) & 0xFF
            for k, fun in self.keydict.items():
                if key == ord(k):
                    fun()


if __name__ == '__main__':
    import sys

    images_dir = sys.argv[1]

    ImageGenerator(images_dir).start()



import cv2

from skcvideo.colors import WHITE
from skcvideo.core import Button
from skcvideo.utils import put_text


class Table(object):
    def __init__(self, box=None, data=None):
        if data is None:
            data = []
        if box is None:
            box = [0, 0, 1920, 1080]
        self.row_height = 20

        self.box = box
        self.data = data
        self.selected_item = None
        self.item_offset = 0

        x1, y1, x2, y2 = self.box
        self.number_of_rows = int((y2 - y1) / self.row_height)

    def build(self, image):
        self.parent.keydict['8'] = self.up
        self.parent.keydict['2'] = self.down
        self.parent.keydict['\r'] = self.activate

        x1, y1, x2, y2 = self.box
        image[y1:y2, x1:x2] = 0
        if self.selected_item is not None:
            image[
                y1 + self.selected_item * self.row_height : y1 + (self.selected_item + 1) * self.row_height, x1:x2
            ] = 128

        cv2.rectangle(image, (x1, y1), (x2, y2), color=WHITE, thickness=2)

        self.buttons = []
        for i, y in enumerate(range(y1, y2, self.row_height)):
            cv2.line(image, (x1, y), (x2, y), color=WHITE, thickness=1)

            button = Button((x1, y, x2, y + self.row_height), self.select, data=i)
            self.buttons.append(button)

        for i in range(self.number_of_rows):
            if i + self.item_offset >= len(self.data):
                continue
            d = self.data[i + self.item_offset]
            y = int(round(y1 + (i + 0.5) * self.row_height))
            put_text(
                image,
                self.format_text(d),
                (x1 + 5, y),
                fontScale=0.5,
                align_x='left',
                align_y='center',
            )

    def format_text(self, d):
        return d['text']

    def select(self, i, *args, **kwargs):
        self.selected_item = i
        self.build(self.parent.background)

    def up(self):
        if self.selected_item is not None:
            if self.selected_item > 0:
                self.select(self.selected_item - 1)
            elif self.selected_item == 0 and self.item_offset > 0:
                self.item_offset -= 1
                self.build(self.parent.background)
            self.parent._refresh()

    def down(self):
        if self.selected_item is not None:
            if self.selected_item < self.number_of_rows - 1:
                self.select(self.selected_item + 1)
            elif (
                self.selected_item == self.number_of_rows - 1
                and self.item_offset < len(self.data) - self.number_of_rows
            ):
                self.item_offset += 1
                self.build(self.parent.background)
            self.parent._refresh()

    def activate(self):
        """
        Example of usage :

        if self.selected_item is not None:
            self.parent.jump(self.data[self.selected_item]['frame'])
            self.parent._refresh()
        """
        pass

    def refresh(self, image, frame):
        pass



import os

import cv2
import numpy as np


class ImagesListIncrustation(object):
    def __init__(self, box=None, images_paths=None, shuffle=False, maximum_length=None):
        if box is None:
            box = [0, 575, 720, 575 + 1280]
        self.box = box
        self.images_paths = images_paths

        if shuffle:
            indices = np.random.permutation(len(self.images_paths))
            if maximum_length is not None:
                indices = indices[:maximum_length]
            self.images_paths = [self.images_paths[i] for i in indices]
        elif maximum_length is not None:
            self.images_paths = self.images_paths[:maximum_length]

        self.image_name_to_i = {}
        for i, image_path in enumerate(self.images_paths):
            image_name = os.path.splitext(os.path.basename(image_path))[0]
            self.image_name_to_i[image_name] = i

        self.images_list = []
        for image_path in self.images_paths:
            image = cv2.imread(image_path)
            self.images_list.append(image)

    @property
    def max_frame(self):
        return len(self.images_list) - 1

    def build(self, *args, **kwargs):
        pass

    def get_display_box(self, image):
        y1, x1, y2, x2 = self.box
        box_height, box_width = y2 - y1, x2 - x1
        im_height, im_width = image.shape[:2]

        ratio = min(box_height / im_height, box_width / im_width)
        new_im_height, new_im_width = int(im_height * ratio), int(im_width * ratio)
        y_offset, x_offset = y1 + int((box_height - new_im_height) / 2), x1 + int((box_width - new_im_width) / 2)
        return y_offset, x_offset, y_offset + new_im_height, x_offset + new_im_width

    def incrust_image(self, big_image, image, frame):
        im_height, im_width = image.shape[:2]
        y1, x1, y2, x2 = self.get_display_box(image)

        if im_height != (y2 - y1) or im_width != (x2 - x1):
            image = cv2.resize(image, (x2 - x1, y2 - y1))
        image = self.process_displayed_image(image, frame)
        big_image[y1:y2, x1:x2, :] = image

    def process_original_image(self, image, frame):
        return image

    def process_displayed_image(self, image, frame):
        return image

    def refresh(self, big_image, frame):
        image = self.images_list[frame].copy()
        image = self.process_original_image(image, frame)
        self.incrust_image(big_image, image, frame)

    def get_image_path(self, frame):
        return self.images_paths[frame]

    def get_frame(self, image_name):
        return self.image_name_to_i.get(image_name, None)

    def convert_to_original_coordinates(self, x, y, frame):
        image = self.images_list[frame]
        y1, x1, y2, x2 = self.get_display_box(image)
        h, w = image.shape[:2]
        x = int(np.round((x - x1) / (x2 - x1) * w))
        y = int(np.round((y - y1) / (y2 - y1) * h))
        x = min(max(x, 0), w - 1)
        y = min(max(y, 0), h - 1)
        return x, y

    def convert_to_display_coordinates(self, x, y, frame):
        image = self.images_list[frame]
        y1, x1, y2, x2 = self.get_display_box(image)
        h, w = image.shape[:2]
        x = int(np.round(x / w * (x2 - x1) + x1))
        y = int(np.round(y / h * (y2 - y1) + y1))
        return x, y



import cv2
import numpy as np

from skcvideo.colors import RED, WHITE
from skcvideo.core import Button
from skcvideo.utils import put_text


class Timeline(object):
    def __init__(self, box=None, timeline_width=20, margin=5, pixel_per_frame=1):
        if box is None:
            box = [955, 110, 1035, 1910]
        self.name = 'Timeline'
        self.box = box
        self.pixel_per_frame = pixel_per_frame

        self.timeline_width = timeline_width
        self.margin = margin
        self.gap = self.timeline_width + 2 * self.margin

        self.timeline_length = self.box[3] - self.box[1]

        self.hitbox = (self.box[1], self.box[0], self.box[3], self.box[2])
        self.buttons = [Button(self.hitbox, self.jump_event)]

    @property
    def min_frame(self):
        return getattr(self.parent, 'min_frame', 0)

    @property
    def max_frame(self):
        return getattr(self.parent, 'max_frame', 9000)

    @property
    def n_timelines(self):
        return ((self.max_frame - self.min_frame) * self.pixel_per_frame) // self.timeline_length + 1

    def jump_event(self, x, y, *kwargs):
        frame = self.get_frame(x, y)
        self.parent.jump(frame)

    def timeline_color(self, frame):
        """
        Here you define the color of the timeline with repect to the frame.
        """
        return (0, 0, 0)

    def build(self, image):
        """
        Draws the timeline's background composed of several timeline lines
        with box and graduations.
        """
        # Puts time labels label
        put_text(
            img=image,
            text='min',
            org=(self.box[1] - 90, self.box[0] - self.margin - self.timeline_width // 2),
            fontScale=0.6,
            align_x='left',
        )

        # Draws graduations labels
        for frame in range(0, self.box[3] - self.box[1], 100):
            if (frame + 1) % 600 == 0:
                put_text(
                    img=image,
                    text='{}min'.format((frame + 1) // 600),
                    org=(self.box[1] + frame, self.box[0] - self.margin - self.timeline_width // 2 - 2),
                    fontScale=0.6,
                    align_x='center',
                )
            else:
                put_text(
                    img=image,
                    text='{}s'.format((frame + 1) % 600 // 10),
                    org=(self.box[1] + frame, self.box[0] - self.margin - self.timeline_width // 2 + 4),
                    fontScale=0.4,
                    align_x='center',
                )

        # Draws each timeline's line
        for i in range(self.n_timelines):
            self.draw_timeline_box(image, i)

        # Draws graduations
        for frame in range(self.max_frame - self.min_frame):
            x = frame % self.timeline_length
            y = frame // self.timeline_length

            # A small mark every 5 seconds
            if ((frame + 1) % 50) == 0:
                cv2.line(
                    image,
                    (self.box[1] + x, self.box[0] + y * self.gap + self.margin - 1),
                    (self.box[1] + x, self.box[0] + (y + 1) * self.gap - self.margin + 1),
                    color=WHITE,
                    thickness=1,
                )

            # A big mark every minute
            if ((frame + 1) % 600) == 0:
                cv2.line(
                    image,
                    (self.box[1] + x, self.box[0] + y * self.gap + self.margin - 3),
                    (self.box[1] + x, self.box[0] + (y + 1) * self.gap - self.margin + 3),
                    color=WHITE,
                    thickness=2,
                )

        self.draw_timeline_data(image)

    def refresh(self, image, frame):
        self.draw_timer(image, frame)

    def draw_timeline_box(self, image, i):
        """
        Draws one line of the timeline's background, which consists in a
        simple white box.
        """
        # Manage the offset
        y_min = self.box[0] + self.margin + i * self.gap
        y_max = y_min + self.timeline_width

        # The last box may not go up to the end.
        if i == self.n_timelines - 1:
            frame_max = (self.max_frame - self.min_frame) * self.pixel_per_frame % self.timeline_length
            timeline_max = self.box[1] + frame_max
        else:
            timeline_max = self.box[3]

        # Draws the box
        cv2.rectangle(
            image,
            (self.box[1], y_min),
            (timeline_max, y_max),
            color=WHITE,
            thickness=1,
        )

        # Adds a time label before the line
        put_text(
            img=image,
            text='{}-{}'.format(3 * i, 3 * (i + 1)),
            org=(self.box[1] - 90, (y_min + y_max) // 2),
            fontScale=0.6,
            align_x='left',
        )

    def draw_timeline_data(self, im):
        """
        Draws information on the timeline. Useful to have a global view of
        your data or to have a reference for jumping in the video.
        """
        for frame in range(self.min_frame, self.max_frame):
            self.draw_one_timeline_data(im, frame)

    def draw_one_timeline_data(self, im, frame):
        """
        Colors the given frame on the timeline according to the timeline_color
        function
        """
        color = self.timeline_color(frame)
        if color != (0, 0, 0):
            timer = frame - self.min_frame
            timer_x = timer % self.timeline_length
            timer_y = timer // self.timeline_length

            x1 = self.box[1] + timer_x * self.pixel_per_frame
            x2 = x1 + self.pixel_per_frame
            y1 = timer_y * self.gap + self.box[0] + self.margin + 1
            y2 = y1 + self.timeline_width - 2
            im[y1:y2, x1:x2] = color

    def draw_timer(self, image, frame):
        """
        Draws a timer on the timeline on the given frame. To be used in the
        refresh method of the parent Reader class.
        """
        timer = (frame - self.min_frame) * self.pixel_per_frame
        timer_x = timer % self.timeline_length
        timer_y = timer // self.timeline_length
        cv2.line(
            image,
            (self.box[1] + timer_x, timer_y * self.gap + self.box[0]),
            (self.box[1] + timer_x, (timer_y + 1) * self.gap + self.box[0]),
            color=WHITE,
            thickness=3,
        )
        cv2.line(
            image,
            (self.box[1] + timer_x, timer_y * self.gap + self.box[0]),
            (self.box[1] + timer_x, (timer_y + 1) * self.gap + self.box[0]),
            color=RED,
            thickness=2,
        )

    def get_frame(self, x, y):
        """
        Returns the frame corresponding to a given pixel of the timeline.
        It is used to be able to click one the timeline to jump to a
        particular frame.
        """
        x = x - self.box[1]
        y = (y - self.box[0]) // self.gap
        frame = (x + y * self.timeline_length) // self.pixel_per_frame + self.min_frame
        return frame


GRAY_BAR = np.array(
    [
        [189, 189, 190],
        [193, 193, 194],
        [196, 197, 198],
        [200, 201, 202],
        [204, 205, 206],
        [208, 208, 209],
        [212, 212, 213],
        [215, 216, 217],
        [219, 220, 221],
        [235, 235, 236],
    ],
)

BLUE_BAR = np.array(
    [
        [224, 137, 44],
        [218, 134, 43],
        [213, 130, 42],
        [207, 127, 41],
        [204, 125, 40],
        [204, 125, 40],
        [204, 125, 40],
        [204, 125, 40],
    ],
)


class VlcTimeline(object):
    def __init__(self, box=None):
        """
        Args:
            box: list [x1, y1, x2, y2]
        """
        if box is None:
            box = [79, 966, 1771, 976]
        self.box = box
        self.buttons = [Button(self.box, self.jump_event)]
        self.timeline_length = float(self.box[2] - self.box[0])

    @property
    def min_frame(self):
        return getattr(self.parent, 'min_frame', 0)

    @property
    def max_frame(self):
        return getattr(self.parent, 'max_frame', 9000)

    @property
    def frames_length(self):
        return float(self.max_frame - self.min_frame)

    def jump_event(self, x, y, *kwargs):
        frame = self.get_frame(x, y)
        self.parent.jump(frame)

    def build(self, image):
        image[self.box[1] : self.box[1] + 18] = np.array([240, 241, 242])[np.newaxis, np.newaxis, :]
        image[self.box[1] + 4 : self.box[1] + 14, self.box[0] : self.box[2]] = GRAY_BAR[:, np.newaxis, :]

    def refresh(self, image, frame):
        self.draw_timer(image, frame)

    def draw_timer(self, image, frame):
        frame = float(frame - self.min_frame)
        i = int(np.round(frame / self.frames_length * self.timeline_length))
        image[self.box[1] + 5 : self.box[1] + 13, self.box[0] : self.box[0] + i] = BLUE_BAR[:, np.newaxis, :]

    def get_frame(self, x, y):
        x = float(x - self.box[0])
        frame = int(np.round(x / self.timeline_length * self.frames_length)) + self.min_frame
        return frame



import os
import sys

import cv2
import imageio
import numpy as np

from skcvideo.minimap import Minimap
from skcvideo.timeline import VlcTimeline
from skcvideo.utils import put_text
from skcvideo.video_capture import StoredImagesVideoCapture


class Button(object):
    """
    Used to define a clickable button on the image executing a given callback
    when cliked. Some data specifying the button can be passed at the object
    creation.

    Args:
        - hitbox: tuple (x1, y1, x2, y2) the bounding box of the clickable area.
        - callback: a function taking x, y (the coordinates of the click) and
          optionnaly data as arguments.
        - data (optionnal): data of any shape that will be used by the callback.
    """

    def __init__(self, hitbox, callback, data=None):
        self.hitbox = hitbox
        self.data = data
        self.given_callback = callback

    def callback(self, *kwargs):
        if self.data is None:
            return self.given_callback(*kwargs)
        else:
            return self.given_callback(self.data, *kwargs)


class Reader(StoredImagesVideoCapture):
    """
    A video displayer that allows interaction with the image by using buttons
    or keyboard.

    The main advantage of this displayer is that it allows to read the video
    backward while keeping relatively fast.

    The best way to use this displayer is to make your own class inheriting
    from this one and overridding its methods.
    """

    def __init__(self, video_path, timeline=None, **kwargs):
        super(Reader, self).__init__(video_path, colormap='bgr', **kwargs)

        self.to_exit = False

        # The key/function mapping
        self.keydict = {
            'k': self.next,
            'j': self.previous,
            'q': self.exit,
        }

        # The clickable buttons
        self.buttons = []

        if not hasattr(self, 'video_incrustation'):
            self.video_incrustation = None

        if not hasattr(self, 'minimap'):
            self.minimap = Minimap(
                box=[200, 0, 590, 575],
                pitch_length=105.0,
                pitch_width=68.0,
            )

        if not hasattr(self, 'timeline'):
            self.timeline = VlcTimeline(
                box=[79, 1771],
                parent=self,
            )

        self.buttons.append(Button(self.timeline.hitbox, self.jump_event))

        self.background = self.create_background()

        self.init(video_path, **kwargs)

        self._refresh()

        cv2.namedWindow('image', cv2.WINDOW_NORMAL)
        cv2.resizeWindow('image', 1280, 720)
        cv2.setMouseCallback('image', self.click_event)

    def init(self, video_path, **kwargs):
        pass

    @property
    def image_to_disp(self):
        """
        This property specifies the image to be displayed. You would override
        it at your convenience e.g. to only display a subpart of the global
        image.
        """
        return self.big_image

    def exit(self):
        self.to_exit = True

    def create_background(self):
        """
        Here you define the elements of the image that don't change throughout
        the video or manipulations.
        """
        im = np.zeros((980, 1855, 3), dtype=np.uint8)
        if self.video_incrustation is not None:
            self.video_incrustation.build(im)
        self.minimap.build(im)
        self.timeline.build(im)
        im = self.draw_background(im)
        return im

    def draw_background(self, im):
        return im

    def timeline_color(self, frame):
        """
        Here you define the color of the timeline with repect to the frame.
        """
        return (0, 0, 0)

    def jump_event(self, x, y, *kwargs):
        frame = self.timeline.get_frame(x, y)
        self.jump(frame)

    def click_event(self, event, x, y, flags, param):
        """
        Part of the core engine that manages the buttons.

        /!\\ Should not be overridden without knowing what you do.
        """
        if event == cv2.EVENT_LBUTTONUP:
            if hasattr(self, 'buttons'):
                for button in self.buttons:
                    x1, y1, x2, y2 = button.hitbox
                    if x1 < x < x2 and y1 < y < y2:
                        button.callback(x, y)
            self._refresh()

    def _refresh(self):
        """
        Here you define the appearance of the image to be displayed with
        respect to structural elements such as the frame index.

        It is called each time the user is interacting with the image
        (clicks, keys, previous, next, ...) to allow updating it with new
        information.
        """
        self.big_image = self.background.copy()
        self.refresh()

    def refresh(self):
        put_text(
            img=self.big_image,
            text='Frame {}'.format(self.frame),
            org=(20, 20),
            align_x='left',
            align_y='top',
        )
        self.timeline.refresh(self.big_image, self.frame)
        if self.video_incrustation is not None:
            self.video_incrustation.refresh(self.big_image, self.image.copy())
        self.minimap.refresh(self.big_image, [])

    def start(self):
        """
        Part of the core engine that manages the display of the image and the
        keys.

        /!\\ Should not be overridden without knowing what you do.
        """
        while not self.to_exit:
            cv2.imshow('image', self.image_to_disp)
            key = cv2.waitKey(1) & 0xFF
            for k, fun in self.keydict.items():
                if key == ord(k):
                    fun()

    def create_video(self, video_path='video.mp4'):
        if os.path.exists(video_path):
            print('video_path already exists, overwite (y/n)?')
            answer = input()
            if answer.lower() != 'y':
                return
        video = imageio.get_writer(video_path, 'ffmpeg', fps=10, quality=5.5)
        print('Creating video...')
        for frame in range(self.min_frame, self.max_frame):
            sys.stdout.write('\r{}/{}'.format(frame - self.min_frame, self.max_frame - self.min_frame - 1))
            sys.stdout.flush()
            self.seek(frame)
            self._refresh()
            video.append_data(cv2.cvtColor(self.big_image, cv2.COLOR_BGR2RGB))
        sys.stdout.write('\n')
        sys.stdout.flush()
        print('Done')
        video.close()


if __name__ == '__main__':
    video_path = sys.argv[1]
    reader = Reader(video_path, fps=10)
    reader.start()



import cv2
import numpy as np

from skcvideo.video_capture import StoredImagesVideoCapture


class VideoIncrustation(StoredImagesVideoCapture):
    def __init__(self, video_path, box=None, **kwargs):
        if box is None:
            box = [0, 575, 720, 575 + 1280]
        self.box = box
        colormap = kwargs.get('colormap', 'bgr')
        super(VideoIncrustation, self).__init__(video_path, colormap=colormap, **kwargs)
        self.max_frame = self.get_max_frame()

    def build(self, *args, **kwargs):
        pass

    def get_image(self, frame):
        if frame >= self.max_frame:
            y1, x1, y2, x2 = self.box
            box_height, box_width = y2 - y1, x2 - x1
            image = np.zeros((box_height, box_width, 3), dtype=np.uint8)
        else:
            image = self.seek(frame).copy()
        return image

    def process_image(self, image, frame):
        return image

    def incrust_image(self, big_image, image):
        y1, x1, y2, x2 = self.box
        box_height, box_width = y2 - y1, x2 - x1
        im_height, im_width = image.shape[:2]
        if im_height != box_height or im_width != box_width:
            image = cv2.resize(image, (box_width, box_height))
        big_image[y1:y2, x1:x2, :] = image

    def refresh(self, big_image, frame):
        image = self.get_image(frame)
        image = self.process_image(image, frame)
        self.incrust_image(big_image, image)



import numpy as np

CIRCLE_RADIUS = 9.15
GOAL_SIZE = 7.32
GOAL_AREA_SIZE = 5.5
PENALTY_AREA_SIZE = 16.5
PENALTY_MARK_DISTANCE = 11.0


# This file contains all the information concerning the soccer field model.


def create_field_objects(pitch_length=105.0, pitch_width=68.0):
    arc_angle = np.arccos((PENALTY_AREA_SIZE - PENALTY_MARK_DISTANCE) / CIRCLE_RADIUS)
    field_objects = {
        'Center Circle': {
            'type': 'circle',
            'x': 0.0,
            'y': 0.0,
            'radius': CIRCLE_RADIUS,
            'startAngle': 0.0,
            'endAngle': 2.0 * np.pi,
        },
        'Halfway line': {
            'type': 'line',
            'start_point': (0.0, -pitch_width / 2.0),
            'end_point': (0.0, pitch_width / 2.0),
        },
    }

    for level, level_sign in zip(['Lower', 'Upper'], [-1.0, 1.0]):
        x = pitch_length / 2.0
        y = pitch_width / 2.0
        field_objects['{} touch line'.format(level)] = {
            'type': 'line',
            'start_point': (
                -x,
                level_sign * y,
            ),
            'end_point': (
                x,
                level_sign * y,
            ),
        }

    for side, sign in zip(['Right', 'Left'], [1.0, -1.0]):
        x = pitch_length / 2.0
        y = pitch_width / 2.0
        field_objects['{} goal line'.format(side)] = {
            'type': 'line',
            'start_point': (
                sign * x,
                -y,
            ),
            'end_point': (
                sign * x,
                y,
            ),
        }

        x = pitch_length / 2.0 - PENALTY_AREA_SIZE
        y = GOAL_SIZE / 2.0 + PENALTY_AREA_SIZE
        field_objects['{} penalty line'.format(side)] = {
            'type': 'line',
            'start_point': (
                sign * x,
                -y,
            ),
            'end_point': (
                sign * x,
                y,
            ),
        }

        x = pitch_length / 2.0 - GOAL_AREA_SIZE
        y = GOAL_SIZE / 2.0 + GOAL_AREA_SIZE
        field_objects['{} goal area line'.format(side)] = {
            'type': 'line',
            'start_point': (
                sign * x,
                -y,
            ),
            'end_point': (
                sign * x,
                y,
            ),
        }

        field_objects['{} penalty arc'.format(side)] = {
            'type': 'circle',
            'x': sign * (pitch_length / 2.0 - PENALTY_MARK_DISTANCE),
            'y': 0.0,
            'radius': CIRCLE_RADIUS,
            'startAngle': -arc_angle - (sign + 1.0) * np.pi / 2.0,
            'endAngle': arc_angle - (sign + 1.0) * np.pi / 2.0,
        }

        for level, level_sign in zip(['lower', 'upper'], [-1.0, 1.0]):
            y = GOAL_SIZE / 2.0 + PENALTY_AREA_SIZE
            field_objects['{} {} penalty line'.format(side, level)] = {
                'type': 'line',
                'start_point': (
                    sign * pitch_length / 2.0,
                    level_sign * y,
                ),
                'end_point': (
                    sign * (pitch_length / 2.0 - PENALTY_AREA_SIZE),
                    level_sign * y,
                ),
            }

            y = GOAL_SIZE / 2.0 + GOAL_AREA_SIZE
            field_objects['{} {} goal area line'.format(side, level)] = {
                'type': 'line',
                'start_point': (
                    sign * pitch_length / 2.0,
                    level_sign * y,
                ),
                'end_point': (
                    sign * (pitch_length / 2.0 - GOAL_AREA_SIZE),
                    level_sign * y,
                ),
            }
    return field_objects



import cv2

from skcvideo.colors import WHITE
from skcvideo.core import Button
from skcvideo.utils import put_text


class ButtonWidget(object):
    def __init__(self, box, text, callback, data=None, color=WHITE, pressed_color=None):
        self.box = box
        self.text = text
        self._callback = callback
        self._is_pressed = False
        self.default_color = color
        if pressed_color is None:
            self.pressed_color = color
        else:
            self.pressed_color = pressed_color
        self.buttons = [Button(box, callback, data=data)]

    @property
    def is_pressed(self):
        return self._is_pressed

    def build(self, im):
        x1, y1, x2, y2 = self.box
        xc, yc = (x1 + x2) // 2, (y1 + y2) // 2

        if self.is_pressed:
            color = self.pressed_color
            thickness = 2
        else:
            color = self.default_color
            thickness = 1

        im = cv2.rectangle(im, (x1, y1), (x2, y2), color, thickness)
        put_text(im, self.text, (xc, yc), color=color)
        return im

    def refresh(self, image, frame):
        pass

    def callback(self, *args, **kwargs):
        self._is_pressed = not self._is_pressed
        return self._callback(*args, **kwargs)



import cv2
import numpy as np

from skcvideo.colors import WHITE
from skcvideo.field_model import create_field_objects
from skcvideo.utils import put_text

FIELD_COLOR = (65, 165, 113)


class Minimap(object):
    def __init__(self, box=None, pitch_length=105.0, pitch_width=68.0):
        if box is None:
            box = [200, 57, 740, 57 + 796]
        self.box = box
        self.pitch_length = pitch_length
        self.pitch_width = pitch_width

        self.xc, self.yc = (self.box[1] + self.box[3]) / 2.0, (self.box[0] + self.box[2]) / 2.0
        self.w, self.h = self.box[3] - self.box[1], self.box[2] - self.box[0]
        self.pixel_per_meter = 5.0 * min(float(self.h) / 390.0, float(self.w) / 575.0)

    def build(self, image):
        image[self.box[0] : self.box[2], self.box[1] : self.box[3], :] = np.array(FIELD_COLOR, dtype=np.uint8)[
            np.newaxis, np.newaxis, :
        ]

        field_objects = create_field_objects(
            pitch_length=self.pitch_length,
            pitch_width=self.pitch_width,
        )

        for _name, field_object in field_objects.items():
            if field_object['type'] == 'line':
                start_x, start_y = self.switch_coords_meter_to_minimap(*field_object['start_point'])
                end_x, end_y = self.switch_coords_meter_to_minimap(*field_object['end_point'])
                cv2.line(
                    img=image,
                    pt1=(start_x, start_y),
                    pt2=(end_x, end_y),
                    color=WHITE,
                    thickness=2,
                )
            elif field_object['type'] == 'circle':
                x, y = self.switch_coords_meter_to_minimap(
                    field_object['x'],
                    field_object['y'],
                )
                radius = int(np.round(self.pixel_per_meter * field_object['radius']))
                startAngle = int(np.round(180.0 * field_object['startAngle'] / np.pi))
                endAngle = int(np.round(180.0 * field_object['endAngle'] / np.pi))

                cv2.ellipse(
                    img=image,
                    center=(x, y),
                    axes=(radius, radius),
                    angle=0,
                    startAngle=startAngle,
                    endAngle=endAngle,
                    color=WHITE,
                    thickness=2,
                )

    def switch_coords_meter_to_minimap(self, x, y):
        x_to_disp = np.round(x * self.pixel_per_meter + self.xc)
        y_to_disp = np.round(-y * self.pixel_per_meter + self.yc)
        x_to_disp, y_to_disp = map(int, [x_to_disp, y_to_disp])
        return x_to_disp, y_to_disp

    def refresh(self, image, data, info_mapping=lambda d: d):
        """
        info_mapping: a function taking d and returning a dict containing:
            {
                'x': float,
                'y': float,
                'radius': int,
                'color': (int, int, int),
                'second_color': (int, int, int),
                'text': str or None,
                'text_color': (int, int, int) or None,
            }
        """
        for d in data:
            info = info_mapping(d)
            x, y = self.switch_coords_meter_to_minimap(info['x'], info['y'])
            cv2.circle(image, (x, y), info['radius'], info['color'], thickness=-1)
            cv2.circle(image, (x, y), info['radius'], info['second_color'], thickness=1)
            if info['text'] is not None:
                put_text(
                    img=image,
                    text=info['text'],
                    org=(x, y),
                    fontScale=0.4,
                    color=info['text_color'],
                    align_x='center',
                    align_y='center',
                )



import cv2

from skcvideo.colors import WHITE

DEFAULT_FONT = cv2.FONT_HERSHEY_SIMPLEX


def put_text(
    img,
    text,
    org,
    fontFace=DEFAULT_FONT,
    fontScale=1.0,
    color=WHITE,
    thickness=1,
    lineType=cv2.LINE_AA,
    align_x='center',
    align_y='center',
):
    x, y = org
    (w, h), _ = cv2.getTextSize(text, fontFace, fontScale, thickness)

    if align_x == 'left':
        org_x = x
    elif align_x == 'center':
        org_x = x - w // 2
    elif align_x == 'right':
        org_x = x - w

    if align_y == 'bottom':
        org_y = y
    elif align_y == 'center':
        org_y = y + h // 2
    elif align_y == 'top':
        org_y = y + h

    cv2.putText(
        img=img,
        text=text,
        org=(org_x, org_y),
        fontFace=fontFace,
        fontScale=fontScale,
        color=color,
        thickness=thickness,
        lineType=lineType,
    )



import os
import sys
import time

import cv2
import imageio
import numpy as np

from skcvideo.colors import BLACK
from skcvideo.utils import put_text


class Button(object):
    """
    Used to define a clickable button on the image executing a given callback
    when cliked. Some data specifying the button can be passed at the object
    creation.

    Args:
        - hitbox: tuple (x1, y1, x2, y2) the bounding box of the clickable area.
        - callback: a function taking x, y (the coordinates of the click) and
          optionnaly data as arguments.
        - data (optionnal): data of any shape that will be used by the callback.
    """

    def __init__(self, hitbox, callback, data=None):
        self.hitbox = hitbox
        self.data = data
        self.given_callback = callback

    def callback(self, *kwargs):
        if self.data is None:
            return self.given_callback(*kwargs)
        else:
            return self.given_callback(self.data, *kwargs)


class Reader(object):
    """
    A video displayer that allows interaction with the image by using buttons
    or keyboard.

    The main advantage of this displayer is that it allows to read the video
    backward while keeping relatively fast.

    The best way to use this displayer is to make your own class inheriting
    from this one and overridding its methods.
    """

    def __init__(self, *args, **kwargs):
        self.to_exit = False
        self.size = kwargs.get('size', (1920, 1080))
        self.min_frame = kwargs.get('min_frame', 0)
        self.max_frame = kwargs.get('max_frame', 9000)
        self.frame = self.min_frame

        self.is_playing = False
        self.max_playing_fps = kwargs.get('max_playing_fps', 10.0)
        self.playing_fps = None

        # The key/function mapping
        self.keydict = {
            'k': self.next,
            'j': self.previous,
            'q': self.exit,
            'p': self.toggle_is_playing,
            ' ': self.toggle_is_playing,
        }

        # Widgets (the order of the widgets defines the order in which they
        # will be drawn)
        self.widgets = []
        for widget in kwargs.get('widgets', []):
            self.add_widget(widget)

        # The clickable buttons
        self.buttons = []

        self.background = self.build()

        self._refresh()

    @property
    def image_to_disp(self):
        """
        This property specifies the image to be displayed. You would override
        it at your convenience e.g. to only display a subpart of the global
        image.
        """
        return self.big_image

    def toggle_is_playing(self):
        self.is_playing = not self.is_playing

    def next(self):
        if self.frame < self.max_frame:
            self.frame += 1
        self._refresh()

    def previous(self):
        if self.frame > self.min_frame:
            self.frame -= 1
        self._refresh()

    def jump(self, frame):
        if frame < self.min_frame:
            self.frame = self.min_frame
        elif frame >= self.max_frame:
            self.frame = self.max_frame - 1
        else:
            self.frame = frame

    def exit(self):
        self.to_exit = True

    def add_widget(self, widget):
        self.widgets.append(widget)
        widget.parent = self

    def build(self):
        """
        Here you define the elements of the image that don't change throughout
        the video or manipulations.
        """
        im = np.zeros((self.size[1], self.size[0], 3), dtype=np.uint8)
        for widget in self.widgets:
            widget.build(im)
        return im

    def click_event(self, event, x, y, flags, param):
        """
        Part of the core engine that manages the buttons.

        /!\\ Should not be overridden without knowing what you do.
        """
        if event == cv2.EVENT_LBUTTONUP:
            for button in [b for widget in [self] + self.widgets for b in getattr(widget, 'buttons', [])]:
                x1, y1, x2, y2 = button.hitbox
                if x1 <= x < x2 and y1 <= y < y2:
                    button.callback(x, y)
            self._refresh()

    def _refresh(self):
        """
        Here you define the appearance of the image to be displayed with
        respect to structural elements such as the frame index.

        It is called each time the user is interacting with the image
        (clicks, keys, previous, next, ...) to allow updating it with new
        information.
        """
        self.big_image = self.background.copy()
        self.refresh()

    def refresh(self):
        for widget in self.widgets:
            widget.refresh(self.big_image, self.frame)

        put_text(
            img=self.big_image,
            text='Frame {}'.format(self.frame),
            org=(20, 20),
            align_x='left',
            align_y='top',
            color=BLACK,
            thickness=3,
        )
        put_text(
            img=self.big_image,
            text='Frame {}'.format(self.frame),
            org=(20, 20),
            align_x='left',
            align_y='top',
        )
        if self.is_playing and self.playing_fps is not None:
            put_text(
                img=self.big_image,
                text='fps: {:.2f}'.format(self.playing_fps),
                org=(1900, 20),
                align_x='right',
                align_y='top',
            )

    def start(self):
        """
        Part of the core engine that manages the display of the image and the
        keys.

        /!\\ Should not be overridden without knowing what you do.
        """
        cv2.namedWindow('image', cv2.WINDOW_NORMAL)
        cv2.resizeWindow('image', 1280, 720)
        cv2.setMouseCallback('image', self.click_event)

        last_time = None
        while not self.to_exit:
            cv2.imshow('image', self.image_to_disp)
            key = cv2.waitKey(1) & 0xFF

            for k, fun in self.keydict.items():
                if key == ord(k):
                    fun()

            if self.is_playing:
                if last_time is not None:
                    spent_time = time.time() - last_time
                    if spent_time < 1 / self.max_playing_fps:
                        time.sleep(1 / self.max_playing_fps - spent_time)

                    self.playing_fps = 1 / (time.time() - last_time)
                last_time = time.time()
                self.next()

    def create_video(self, video_path='video.mp4', min_frame=None, max_frame=None, force_overwrite=False):
        if not force_overwrite and os.path.exists(video_path):
            print('video_path already exists, overwite (y/n)?')
            answer = input()
            if answer.lower() != 'y':
                return
        video = imageio.get_writer(video_path, 'ffmpeg', fps=10, quality=5.5)
        print('Creating video...')
        if min_frame is None:
            min_frame = self.min_frame
        if max_frame is None:
            max_frame = self.max_frame
        for frame in range(min_frame, max_frame):
            sys.stdout.write('\r{}/{}'.format(frame - min_frame, max_frame - min_frame - 1))
            sys.stdout.flush()
            self.frame = frame
            self._refresh()
            video.append_data(cv2.cvtColor(self.big_image, cv2.COLOR_BGR2RGB))
        sys.stdout.write('\n')
        sys.stdout.flush()
        print('Done')
        video.close()


if __name__ == '__main__':
    reader = Reader()
    reader.start()



import logging
import math

import cv2
from ffmphisdp.pyav_reader import ControlledVideoReader as ControlledFPSVideoCapture_ffmphisdp


class ControlledFPSVideoCapture(object):
    FRAME_SELECTION_LEGACY = 'legacy'
    FRAME_SELECTION_FFMPHISDP = 'ffmphisdp'

    def __new__(cls, *args, **kwargs):
        kwargs_copy = kwargs.copy()
        frame_selection_version = kwargs_copy.pop(
            'frame_selection_version', ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY
        )
        if frame_selection_version == ControlledFPSVideoCapture.FRAME_SELECTION_LEGACY:
            reader = ControlledFPSVideoCapture_skcvideo(*args, **kwargs_copy)
            reader.frame_selection_version = frame_selection_version
            return reader
        elif frame_selection_version == ControlledFPSVideoCapture.FRAME_SELECTION_FFMPHISDP:

            def set_convert(self, prop_id, value):
                """Convert form global frame index to selection frame index"""
                if prop_id != cv2.CAP_PROP_POS_FRAMES:
                    raise NotImplementedError
                idx_global = self.frame_selection_index_mapping[value]
                print(f'Automatically converting index from {value} to {idx_global} (local to global)')
                self.set_frame_idx(idx_global)

            def get_convert(self, prop_id):
                """convert from global frame index to selection frame index"""
                if prop_id == cv2.CAP_PROP_POS_FRAMES:
                    print(self.current_frame_index)
                    return self.frame_selection_index_mapping.index(self.current_frame_index)
                elif prop_id == cv2.CAP_PROP_FRAME_COUNT:
                    return len(self.frame_selection_index_mapping)
                else:
                    raise NotImplementedError

            ControlledFPSVideoCapture_ffmphisdp.set = set_convert
            ControlledFPSVideoCapture_ffmphisdp.get = get_convert
            reader = ControlledFPSVideoCapture_ffmphisdp(*args, **kwargs_copy)
            reader.frame_selection_version = frame_selection_version
            reader.frame_selection_index_mapping = [
                idx_global for idx_global, selected in enumerate(reader.frame_selection) if selected
            ]
            return reader
        else:
            raise ValueError('Invalid frame_selection_version')


class ControlledFPSVideoCapture_skcvideo(object):
    """
    VideoCapture Reader from any input fps to any output fps.
    Has been tested with output_fps=10 and input_fps=10, 25, 30, 50, 60
    Note that input frame and output frame are the last decoded frames
    While cap.get(cv2.CAP_PROP_POS_FRAMES) is next frame to be read

    This class aims to simulate the result of downsampling by ffmpeg
    directly by reading the original video. This avoid having to store
    multiple versions of the video.
    """

    def __init__(self, *args, **kwargs):
        logger = kwargs.pop('logger', logging.getLogger(__name__))

        self.input_frame = -1
        self.output_frame = -1
        self.output_fps = kwargs.pop('fps', 10)

        self._cap = cv2.VideoCapture(*args, **kwargs)
        # We round fps cause sometimes opencv gives fps like 25.00001429630351 while ffmpeg gives 25
        # Still don't know if we have correct behaviour for weird fps like 29.97
        self.input_fps = round(self._cap.get(cv2.CAP_PROP_FPS))

        if self.output_fps == 'same':
            self.output_fps = self.input_fps

        message = 'Initializing ControlledFPSVideoCapture with input_fps={} and output_fps={}'.format(
            self.input_fps, self.output_fps
        )
        logger.info(message)

    def read(self):
        self.output_frame += 1
        if self.output_fps == self.input_fps:
            self.input_frame += 1
            return self._cap.read()

        # This general case do not work with output_fps == input_fps (weird case output_frame == 2)
        if self.output_frame < 2:
            # Do not understand why but that's what ffmpeg does
            frames_to_get = 1
        else:
            frames_to_get = int(
                math.ceil((self.output_frame + 1) * float(self.input_fps) / self.output_fps)
                - math.ceil((self.output_frame) * float(self.input_fps) / self.output_fps)
            )
            # Do not understand why but that's what ffmpeg does
            if self.output_frame == 2:
                frames_to_get -= 2

        for _ in range(frames_to_get):
            self.input_frame += 1
            ret, im = self._cap.read()
            if ret is False:
                return ret, im

        return ret, im

    def get(self, prop_id):
        if prop_id == cv2.CAP_PROP_BUFFERSIZE:
            raise NotImplementedError
        elif prop_id == cv2.CAP_PROP_FPS:
            return self.output_fps
        elif prop_id == cv2.CAP_PROP_POS_FRAMES:
            return self.output_frame + 1
        elif prop_id == cv2.CAP_PROP_POS_MSEC:
            return 1000 * float(self.output_frame + 1) / self.output_fps
        elif prop_id == cv2.CAP_PROP_FRAME_COUNT:
            return int(math.floor(float(self._cap.get(prop_id)) / self.input_fps * self.output_fps))
        else:
            return self._cap.get(prop_id)

    def set(self, prop_id, value):
        if prop_id in [cv2.CAP_PROP_BUFFERSIZE, cv2.CAP_PROP_FPS, cv2.CAP_PROP_FRAME_COUNT]:
            raise NotImplementedError
        elif prop_id in [cv2.CAP_PROP_POS_FRAMES, cv2.CAP_PROP_POS_MSEC, cv2.CAP_PROP_POS_AVI_RATIO]:
            if prop_id == cv2.CAP_PROP_POS_FRAMES:
                if value < 0:
                    value = 0
                output_frame = value - 1
            elif prop_id == cv2.CAP_PROP_POS_MSEC:
                output_frame = round(float(value * self.output_fps) / 1000) - 1
            elif prop_id == cv2.CAP_PROP_POS_AVI_RATIO:
                output_frame = round(self.get(cv2.CAP_PROP_FRAME_COUNT) * cv2.CAP_PROP_POS_AVI_RATIO) - 1
            self.output_frame = output_frame
            fps_ratio = float(self.input_fps) / self.output_fps
            if fps_ratio == 1.0 or output_frame < 2:
                self.input_frame = self.output_frame
            else:
                self.input_frame = math.floor(output_frame * fps_ratio) - math.floor(fps_ratio + 1)
            return self._cap.set(prop_id, self.input_frame + 1)
        else:
            return self._cap.set(prop_id, value)

    def generator(self):
        while True:
            ret, im = self.read()
            if ret is False:
                raise StopIteration
            yield im

    def release(self):
        return self._cap.release()

    def isOpened(self):
        return self._cap.isOpened()

    def retrieve(self):
        return self._cap.retrieve()

    def open(self, *args, **kwargs):
        """Could be implemented but we don't use it"""
        raise NotImplementedError

    def grab(self):
        """Could be implemented but we don't use it"""
        raise NotImplementedError



import cv2

from skcvideo.controlled_fps_video_capture import ControlledFPSVideoCapture

DEFAULT_WIDTH = 1280
DEFAULT_HEIGHT = 720

SET_READ_TRADEOFF = 20
MAX_STORED = 100
AMONG_FRAMES = 20


class TransformedImageVideoCapture:
    """
    This VideoCapture overlay aims to add operations on the images before
    providing them.

    For instance, the operations are:
        - change channel order to switch from BGR to RGB
        - resize to desired shape

    Additionnaly, you can give an initial frame for the VideoCapture

    Args:
        - min_frame: the initial frame index of the video to start with
        - colormap: the order of the image channels (can be 'rgb' or 'bgr')
        - resize: tuple (width, height) or None: the shape of the outputted
          image. If None, no resize will be applied.
    """

    def __init__(self, *args, **kwargs):
        self.min_frame = kwargs.pop('min_frame', 0)
        self.colormap = kwargs.pop('colormap', 'rgb')
        self.resize = kwargs.pop('resize', (DEFAULT_WIDTH, DEFAULT_HEIGHT))
        self.interlaced = kwargs.pop('interlaced', False)

        if self.min_frame < 0:
            self.min_frame = 0

        if self.colormap not in ['rgb', 'bgr']:
            raise NotImplementedError

        self.frame_reader = ControlledFPSVideoCapture(*args, **kwargs)

        if self.resize is not None:
            resize_log = 'Resize to {}x{}'.format(self.resize[0], self.resize[1])
        else:
            resize_log = 'No resize'
        colormap_log = 'Colormap: {}'.format(self.colormap)
        print('{}, {}'.format(resize_log, colormap_log))

        if self.min_frame > 0:
            self.frame_reader.set(cv2.CAP_PROP_POS_FRAMES, self.min_frame)

        # According to our use-case we prefer to have the first frame already read.
        self.read()

    def read(self):
        ret, self.image = self.frame_reader.read()

        if self.interlaced:
            h, w = self.image.shape[:2]
            self.image = cv2.resize(self.image[::2], (w, h))

        self.image_before_resize = self.image
        if self.resize is not None:
            # ffmpeg prefers to save video with 1088 pixels height, thus we
            # add 8 pixels to our 1080p videos. The following removes those
            # pixels before resizing them to 720p images.
            if self.image.shape[0] == 1088:
                self.image = self.image[8:]

            in_height, in_width = self.image.shape[:2]
            out_width, out_height = self.resize
            if in_height != out_height or in_width != out_width:
                in_aspect_ratio = float(in_height) / float(in_width)
                out_aspect_ratio = float(out_height) / float(out_width)
                if in_aspect_ratio != out_aspect_ratio:
                    size_in = '{}x{}'.format(in_width, in_height)
                    size_out = '{}x{}'.format(out_width, out_height)
                    print('Warning: resizing image with a different aspect ratio')
                    print('    {} -> {}'.format(size_in, size_out))
                self.image = cv2.resize(self.image, self.resize)
        if self.colormap == 'rgb':
            self.image = cv2.cvtColor(self.image, cv2.COLOR_BGR2RGB)

    def set(self, prop_id, value):
        return self.frame_reader.set(prop_id, value)

    def get(self, prop_id):
        return self.frame_reader.get(prop_id)

    def seek(self, frame):
        """
        This method aims to manage the tradoff between directly setting the
        VideoCapture to the desired frame and reading the successive frames
        up to the desired in order to get the frame in the fastest manner.

        /!\\ Default value was chosen empirically and may not be optimal
        depending on your architecture.
        """
        if not (self.frame_reader.output_frame <= frame < self.frame_reader.output_frame + SET_READ_TRADEOFF):
            self.set(cv2.CAP_PROP_POS_FRAMES, frame)
            self.read()

        while self.frame_reader.output_frame < frame:
            self.read()
        return self.image


class StoredImagesVideoCapture(TransformedImageVideoCapture):
    """
    This VideoCapture overlay stores previously read frames for faster access.

    When setting to a new instant in the video, it prepares the previous
    frames by actually setting several frames back in order to facilitate
    backward reading.

    The accessible frames are bounded as it was designed for the displayer and
    we may want a close up on a sequence (in particular for a better precision
    in the timeline).

    Args:
        - min_frame: the index of the first accessible frame
        - max_frame: the index of the last accessible frame
    """

    def __init__(self, *args, **kwargs):
        self.stored_images = {}
        super(StoredImagesVideoCapture, self).__init__(*args, **kwargs)

    def get_max_frame(self):
        return super(StoredImagesVideoCapture, self).get(cv2.CAP_PROP_FRAME_COUNT)

    def read(self):
        """
        Overrides read() method to store the output
        """
        super(StoredImagesVideoCapture, self).read()
        if self.frame_reader.output_frame - MAX_STORED in self.stored_images:
            del self.stored_images[self.frame_reader.output_frame - MAX_STORED]
        self.stored_images[self.frame_reader.output_frame] = self.image

    def set(self, prop_id, value):
        """
        Overrides set() method to erase all the stored values
        """
        if prop_id == cv2.CAP_PROP_POS_FRAMES:
            self.stored_images = {}
            super(StoredImagesVideoCapture, self).set(prop_id, value - AMONG_FRAMES)
        else:
            super(StoredImagesVideoCapture, self).set(prop_id, value)

    def seek(self, frame):
        if frame not in self.stored_images:
            super(StoredImagesVideoCapture, self).seek(frame)
        self.frame = frame
        return self.stored_images[frame]
